import os
import re


class ServiceNameFinder:
    """
    ServiceNameFinder class will help you to find azure services related to dtrg_crm project.
    before you can use this object, the WEBSITE_SITE_NAME env variable should set correctly according to pattern
    which we're using in dtrg team.

    Parameters
    ----------
    kv_name: str
        contains related key vault name.

    psql_host: str
        contains related PostgresSQL host in azure services.

    psql_password_secret_name: str
        related postgres secret name (in key vault) which contains password of postgres admin.
    """

    kv_name = str()
    psql_host = str()
    psql_password_secret_name = str()

    def __init__(self):
        if len(os.getenv('WEBSITE_SITE_NAME').split('-')) != 5:
            raise ValueError('you should set WEBSITE_SITE_NAME correctly. ex: wa-uae-dev-kl-main')

        try:
            app_name = re.sub('wa-', '', os.getenv('WEBSITE_SITE_NAME'))  # uae-dev-kl-main
        except TypeError:
            raise ValueError('you should set WEBSITE_SITE_NAME env variable in order to make ServiceNameFinder work.')

        self.kv_name = f"kv-{app_name}"
        self.psql_host = f"psql-{app_name}.postgres.database.azure.com"
        self.psql_password_secret_name = f"psql-{app_name}-psqladmin-password"



from azure.identity import DefaultAzureCredential
from azure.keyvault.secrets import SecretClient as AzSecretClient, KeyVaultSecret
from os import getenv
from dotenv import load_dotenv


class SecretClient:
    """
    Azure Key Vault Secret Client.

    ```
    Methods
    ----------
    get_secret(secret_name: str, secret_version: str)
        returns KeyVaultSecret object of given secret_name and version.

    get_secrets(secrets: dict)
        looks into secrets dict and puts each key equals to KeyVaultSecret object of given version.

    get_secrets_values(secrets: dict)
        same to get_secrets method but instead of KeyVaultSecret object, you'll have values as str.
    """
    def __init__(self, vault_name: str) -> None:
        """
        Parameters
        ----------
        vault_name : str
            the name of vault you wanna read from.
        """

        load_dotenv()
        vault_url = f'https://{vault_name}.vault.azure.net'
        credential = DefaultAzureCredential()
        self.client = AzSecretClient(vault_url=vault_url, credential=credential)

    def get_secret(self, secret_name: str, secret_version: str = "") -> KeyVaultSecret:
        """
        returns the KeyVaultSecret object belongs to given secret_name and secret_version

        If the argument `version` isn't passed, it will consider last version of secret.

        Parameters
        ----------
        secret_name: str
            name of secret you wanna read.
        secret_version: str, optional
            version of given secret.

        Returns
        -------
        KeyVaultSecret
            contains given secret name result.
        """

        self._check_env_variables()
        return self.client.get_secret(name=secret_name, version=secret_version)

    def get_secrets(self, secrets: dict) -> dict:
        """
        looks into secrets dict and puts each key equals to KeyVaultSecret object of given version.

        Parameters
        ----------
        secrets: dict
            dict object of secret_names and secret_versions that you wanna fetch.
            input dict should be something like this:
            {
                "SECRET_NAME": "SECRET_VERSION",
                "SECOND_SECRET_NAME": "SECOND_SECRET_VERSION"
            }

        Returns
        -------
        dict
            a dict object of secret_names and their values typed KeyVaultSecret.
            NOTE: this method modifies given secrets variable, but also returns result.
        """

        for k, v in secrets.items():
            if type(secrets[k]) == KeyVaultSecret:
                continue
            secrets[k] = self.get_secret(k, v)
        return secrets

    def get_secrets_values(self, secrets: dict) -> dict:
        """
        looks into secrets dict and puts each key equals to str value of given version.

        Parameters
        ----------
        secrets: dict
            dict object of secret_names and secret_versions that you wanna fetch.
            input dict should be something like this:
            {
                "SECRET_NAME": "SECRET_VERSION",
                "SECOND_SECRET_NAME": "SECOND_SECRET_VERSION"
            }

        Returns
        -------
        dict
            a dict object of secret_names and their values typed str.
            NOTE: this method modifies given secrets variable, but also returns result.
        """

        items = self.get_secrets(secrets)
        for k, v in items.items():
            items[k] = v.value
        return items

    @staticmethod
    def _check_env_variables() -> bool:
        """
        check if needed env variable has not been set, then raises an error.

        this env variables represents authentication parameters for azure.

        Returns
        -------
        bool
            returns result as boolean, if all needed variable has been set, then we'll get True.

        Raises
        ------
        ValueError
            this error will happen whenever even one of needed variables has not been set.
        """

        if not getenv('AZURE_CLIENT_ID') or not getenv('AZURE_CLIENT_SECRET') or not getenv('AZURE_TENANT_ID'):
            raise ValueError(''' you should set needed env variables:
                - AZURE_CLIENT_ID
                - AZURE_CLIENT_SECRET
                - AZURE_TENANT_ID
            ''')
        return True



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="azure-integration",  # Replace with your own username
    version="0.0.3",
    author="Mehran Zolghadr",
    author_email="mehran@keyleadhealth.com",
    description="Microsoft Azure Integration",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/IOMehran/azure-integration",
    download_url="https://github.com/IOMehran/azure-integration/archive/refs/tags/v0.0.3.tar.gz",
    project_urls={
        "Bug Tracker": "https://github.com/IOMehran/azure-integration/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",

    install_requires=[
        "azure-common==1.1.27",
        "azure-core==1.13.0",
        "azure-identity==1.5.0",
        "azure-keyvault==4.1.0",
        "azure-keyvault-certificates==4.2.1",
        "azure-keyvault-keys==4.3.1",
        "azure-keyvault-secrets==4.2.0",
        "build==0.3.1.post1",
        "certifi==2020.12.5",
        "cffi==1.14.5",
        "chardet==4.0.0",
        "cryptography==3.4.7",
        "idna==2.10",
        "isodate==0.6.0",
        "msal==1.11.0",
        "msal-extensions==0.3.0",
        "msrest==0.6.21",
        "oauthlib==3.1.0",
        "packaging==20.9",
        "pep517==0.10.0",
        "portalocker==1.7.1",
        "pycparser==2.20",
        "PyJWT==2.0.1",
        "pyparsing==2.4.7",
        "python-dotenv==0.17.0",
        "requests==2.25.1",
        "requests-oauthlib==1.3.0",
        "six==1.15.0",
        "toml==0.10.2",
        "urllib3==1.26.4",
        "gobject"
    ],
)
